function [I_L I_R disparity]=vergence_version_final(filename,baseline,alpha,delta,dx,dy,mode)
% [I_L I_R]=vergence_version_final(filename,baseline,alpha,delta,dx,dy)
%
% filename: name of the file of the range data
% baseline: distance between left and right camera
% alpha: field of view (deg)
% delta: initial direction (deg)
% dx: fixation point in the image coordinates (offset with respect to the  center)
% dy: fixation point in the image coordinates (offset with respect to the  center)
% mode: sequential order of the rotations
%       0 Fick coordinates -> pan-tilt
%       1 Helmholtz coordinates -> tilt-pan
%
% I_L: left view
% I_R: right view
% disparity: struct containing the horizontal (VX) and the vertical (VY)  disparities
%
% Compute the left and the right views and the horizontal and vertical disparities from range data.
% The binocular system has vergent axis. It is possible to specify the fixation point in the 2D image,
% from which the 3D fixation point is computed.
%
% Authors:
% Manuela Chessa and Fabio Solari
% PSPC-lab - Biophysical and Electronic Engineering Department - University of Genoa
% October 2008


if(alpha<0)
    display('alpha must be positive');
    I_L=[];
    I_R=[];
    disparity=[];
    return
end

if(delta-alpha/2<-259/2 || delta+alpha/2>259/2)
    display('out of range');
    I_L=[];
    I_R=[];
    disparity=[];
    return
end


% read data from BRID database
% http://www.dam.brown.edu/ptg/brid/range/index.html
[X Y Z I]=read_brid_data(filename,alpha,delta);

b=baseline/2;
[sy sx]=size(X);

%fixation point in 2D image coordinates
xo=round(sx/2)+dx;
yo=round(sy/2)+dy;

%fixation point in 3D coordinates
Xo=X(yo,xo);
Yo=Y(yo,xo);
Zo=Z(yo,xo);


dZL=Zo;
dZR=Zo;

if (mode ==0)
    %%% pan -tilt
    %pan angles
    if((Xo-X(round(sy/2),round(sx/2)))<b && (Xo-X(round(sy/2),round(sx/2)))>-b)
        dXL=b+(Xo-X(round(sy/2),round(sx/2)));
        dXR=b-(Xo-X(round(sy/2),round(sx/2)));
        WyL=-atan(dXL/dZL);
        WyR=atan(dXR/dZR);
    elseif((Xo-X(round(sy/2),round(sx/2)))<-b)
        dXL=abs((Xo-X(round(sy/2),round(sx/2))))-b;
        dXR=abs((Xo-X(round(sy/2),round(sx/2))))+b;
        WyL=atan(dXL/dZL);
        WyR=atan(dXR/dZR);
    elseif((Xo-X(round(sy/2),round(sx/2)))>b)
        dXL=abs((Xo-X(round(sy/2),round(sx/2))))+b;
        dXR=abs((Xo-X(round(sy/2),round(sx/2))))-b;
        WyL=-atan(dXL/dZL);
        WyR=-atan(dXR/dZR);
    end

    % tilt angle
    dYL=((Yo-Y(round(sy/2),round(sx/2))));
    dYR=((Yo-Y(round(sy/2),round(sx/2))));
    WxL=-atan(dYL/sqrt(dXL^2+dZL^2));
    WxR=-atan(dYR/sqrt(dXR^2+dZR^2));

    % torsion angle
    WzL=0;
    WzR=0;
    
    x_L=-((X+b)*cos(WyL)+Z*sin(WyL))./((X+b)*sin(WyL)*cos(WxL)-Y*sin(WxL)-Z*cos(WyL)*cos(WxL));
    x_R=-((X-b)*cos(WyR)+Z*sin(WyR))./((X-b)*sin(WyR)*cos(WxR)-Y*sin(WxR)-Z*cos(WyR)*cos(WxR));

    y_L=-((X+b)*sin(WyL)*sin(WxL)+Y*cos(WxL)-Z*cos(WyL)*sin(WxL))./((X+b)*sin(WyL)*cos(WxL)-Y*sin(WxL)-Z*cos(WyL)*cos(WxL));
    y_R=-((X-b)*sin(WyR)*sin(WxR)+Y*cos(WxR)-Z*cos(WyR)*sin(WxR))./((X-b)*sin(WyR)*cos(WxR)-Y*sin(WxR)-Z*cos(WyR)*cos(WxR));
    
  
    

elseif (mode ==1)

    %%% tilt-pan
    % pan angles
    dYL=((Yo-Y(round(sy/2),round(sx/2))));
    dYR=((Yo-Y(round(sy/2),round(sx/2))));

    if((Xo-X(round(sy/2),round(sx/2)))<b && (Xo-X(round(sy/2),round(sx/2)))>-b)
        dXL=b+(Xo-X(round(sy/2),round(sx/2)));
        dXR=b-(Xo-X(round(sy/2),round(sx/2)));
        WyL=-atan(dXL/sqrt(dYL^2+dZL^2));
        WyR=atan(dXR/sqrt(dYR^2+dZR^2));
    elseif((Xo-X(round(sy/2),round(sx/2)))<-b)
        dXL=abs((Xo-X(round(sy/2),round(sx/2))))-b;
        dXR=abs((Xo-X(round(sy/2),round(sx/2))))+b;
        WyL=atan(dXL/sqrt(dYL^2+dZL^2));
        WyR=atan(dXR/sqrt(dYR^2+dZR^2));
    elseif((Xo-X(round(sy/2),round(sx/2)))>b)
        dXL=abs((Xo-X(round(sy/2),round(sx/2))))+b;
        dXR=abs((Xo-X(round(sy/2),round(sx/2))))-b;
        WyL=-atan(dXL/sqrt(dYL^2+dZL^2));
        WyR=-atan(dXR/sqrt(dYR^2+dZR^2));
    end

    % tilt angle
    WxL=-atan(dYL/dZL);
    WxR=-atan(dYR/dZR);

    % torsion angle
    WzL=0;
    WzR=0;
    
    x_L=-((X+b)*cos(WyL)+Y*sin(WyL)*sin(WxL)+Z*sin(WyL)*cos(WxL))./((X+b)*sin(WyL)-Y*cos(WyL)*sin(WxL)-Z*cos(WyL)*cos(WxL));
    x_R=-((X-b)*cos(WyR)+Y*sin(WyR)*sin(WxR)+Z*sin(WyR)*cos(WxR))./((X-b)*sin(WyR)-Y*cos(WyR)*sin(WxR)-Z*cos(WyR)*cos(WxR));

    y_L=-(Y*cos(WxL)-Z*sin(WxL))./((X+b)*sin(WyL)-Y*cos(WyL)*sin(WxL)-Z*cos(WyL)*cos(WxL));
    y_R=-(Y*cos(WxR)-Z*sin(WxR))./((X-b)*sin(WyR)-Y*cos(WyR)*sin(WxR)-Z*cos(WyR)*cos(WxR));
end

% roto-translation




x=X./Z;
y=Y./Z;



d_oR.VX=(x_R-x)*sx;
d_oR.VY=(y_R-y)*sy;

d_oL.VX=(x_L-x)*sx;
d_oL.VY=(y_L-y)*sy;

% left and right views
I_R=warp(I,d_oR.VX,d_oR.VY,sx,sy);
I_L=warp(I,d_oL.VX,d_oL.VY,sx,sy);

I_L=I_L./max(max(I_L));
I_R=I_R./max(max(I_R));

%anaglyph
rgb(:,:,1)=I_R;
rgb(:,:,2)=I_L;
rgb(:,:,3)=I_L;
figure, image(rgb)

% vertical and horizontal disparities
disparity.VX=d_oR.VX-d_oL.VX;
disparity.VY=d_oR.VY-d_oL.VY;

%%test disparity gt
I_R_w=warp(I_L,disparity.VX,disparity.VY,sx,sy);
diff=I_R-I_R_w;





%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%

function I=warp(I,vx,vy,sx,sy)

[x y]=meshgrid(1:sx,1:sy);
x=x-vx;
y=y-vy;
I=bilin_interp(I,x,y);

%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%
function r = readrange(infile)
% Reads a range image from the Brown Range Image Database
%
% By Kim S. Pedersen, kimstp@diku.dk, March 15, 2002

fid = fopen(infile,'r','ieee-be');

if fid == -1
    error('Could not open file')
else
    % Read image size
    Rows = fread(fid, 1,'uint16');               % Number of row
    Cols = fread(fid, 1,'uint16');               % Number of columns

    % Read image data
    range = fread(fid, [Rows, Cols], 'uint16');       % Range data
    intensity = fread(fid, [Rows, Cols], 'uint16');   % Intensity data
    bearing = fread(fid, [Rows, Cols], 'uint16');     % Bearing
    inclination = fread(fid, [Rows, Cols], 'uint16'); % Inclination
    fclose(fid);

    r = struct('range', range, 'intensity', intensity, 'bearing', bearing, ...
        'inclination', inclination);
end

%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%
function [X Y Z I] = read_brid_data(input,alpha,delta)

r = readrange(input);
[sy sx]=size(r.range);


start = round((259/2-(delta+alpha/2))/0.18);
stop  = round((259/2-(delta-alpha/2))/0.18);
sx=stop-start;

R=r.range;
I=r.intensity(:,start:stop);

theta=r.bearing*.02*pi/200;
R=R(:,start:stop);
theta=theta(:,start:stop);
alpha=(alpha*pi)/180;
step=0.18*pi/180;
if(mod(sx,2)~=0)
    gamma=pi/2-meshgrid(-alpha/2:step:alpha/2,-1:2/(sy-1):1);
else
    gamma=pi/2-meshgrid(-alpha/2:step:alpha/2+step,-1:2/(sy-1):1);
end


X=R.*sin(theta).*cos(gamma);
Y=R.*cos(theta);
Z=R.*sin(theta).*sin(gamma);

%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%
function I2 = bilin_interp(I1,X,Y)

%
% function I2 = bilin_interp(I1,X,Y)
%
% Arbitrary image rewarping using bilinear interpolation
%
% X (column) and Y (row) (both floating point) are the source locations,
% used to fill the respective pixels
%


[nY1,nX1,rem] = size(I1); % source size
[nY2,nX2] = size(X); % target size

s = size(I1);
s(1:2) = [];

I2 = NaN.*zeros([ nY2 nX2 s ]);

for r = 1:rem

    for x = 1:nX2
        for y = 1:nY2

            % Pixel warping (2x2 group)

            x_w = floor(X(y,x));
            y_w = floor(Y(y,x));

            % Check validity

            if ((x_w>0) & (x_w<nX1) & (y_w>0) & (y_w<nY1))

                xs = X(y,x) - x_w;
                min_xs = 1-xs;
                ys = Y(y,x) - y_w;
                min_ys = 1-ys;

                w_00 = min_xs*min_ys;  % w_xy
                w_10 = xs*min_ys;
                w_01 = min_xs*ys;
                w_11 = xs*ys;

                I2(y,x,r) = w_00*I1(y_w,x_w,r)+ w_10*I1(y_w,x_w+1,r) + w_01*I1(y_w+1,x_w,r) + w_11*I1(y_w+1,x_w+1,r);


            end
        end
    end
end


